/*
 * GStreamer
 * Copyright (C) 2005 Thomas Vander Stichele <thomas@apestaart.org>
 * Copyright (C) 2005 Ronald S. Bultje <rbultje@ronald.bitfreak.net>
 * Copyright (C) 2011 Christian Hemp <<user@hostname.org>>
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Alternatively, the contents of this file may be used under the
 * GNU Lesser General Public License Version 2.1 (the "LGPL"), in
 * which case the following provisions apply instead of the ones
 * mentioned above:
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/**
 * SECTION:element-i2c
 *
 * FIXME:Describe i2c here.
 *
 * <refsect2>
 * <title>Example launch line</title>
 * |[
 * gst-launch -v -m fakesrc ! i2c ! fakesink silent=TRUE
 * ]|
 * </refsect2>
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gst/gst.h>

#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include <linux/types.h>   // für __u32
//#include <linux/videodev2.h>
#include <sys/ioctl.h>

struct v4l2_dbg_match {
  __u32 type;  // Match type
  union {      // Match this chip, meaning determined by type
    __u32 addr;
    char name [32];
  };
} __attribute__ ((packed));


struct v4l2_dbg_register {
  struct v4l2_dbg_match match;
  __u32 size;    // register size in bytes
  __u64 reg;
  __u64 val;
} __attribute__ ((packed));


#undef   VIDIOC_DBG_G_REGISTER
#define  VIDIOC_DBG_G_REGISTER   _IOWR ('V', 80, struct v4l2_dbg_register)
#undef   VIDIOC_DBG_S_REGISTER
#define  VIDIOC_DBG_S_REGISTER    _IOW ('V', 79, struct v4l2_dbg_register)

#define V4L2_CHIP_MATCH_I2C_ADDR   2   // Match against I2C 7-bit address

#include "gsti2c.h"

GST_DEBUG_CATEGORY_STATIC (gst_i2c_debug);
#define GST_CAT_DEFAULT gst_i2c_debug

/* Filter signals and args */
enum
{
  /* FILL ME */
  LAST_SIGNAL
};

enum
{
  ARG_0,
  ARG_DEV,
  ARG_ADDR,
  ARG_REG,
  ARG_MODE,
  ARG_VALUE,
  ARG_FILE,
  ARG_SHOW,
  PROP_0,
  PROP_SILENT
};

#define GST_TYPE_I2C_ACCMODE (gst_i2c_accmode_get_type())
static GType gst_i2c_accmode_get_type (void) {
  static GType type = 0;
  static const GEnumValue data[] = {
    {GST_I2C_ACCESSMODE_SET, "Set Value", "set"},
    {GST_I2C_ACCESSMODE_AND, "And Value", "and"},
    {GST_I2C_ACCESSMODE_OR,  "Or Value",  "or"},
    {GST_I2C_ACCESSMODE_XOR, "Xor Value", "xor"},
    {0, NULL, NULL},
  };

  if (!type)
    type = g_enum_register_static ("GstI2CMode", data);

  return type;
}


/* the capabilities of the inputs and outputs.
 *
 * describe the real formats here.
 */
static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
    GST_PAD_SINK,
    GST_PAD_ALWAYS,
    GST_STATIC_CAPS ("ANY")
    );

static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
    GST_PAD_SRC,
    GST_PAD_ALWAYS,
    GST_STATIC_CAPS ("ANY")
    );

#define _do_init(bla) \
    GST_DEBUG_CATEGORY_INIT (gst_i2c_debug, "i2c", 0, "i2c element");

/*GST_BOILERPLATE (GstI2C, gst_i2c, GstElement,
    GST_TYPE_ELEMENT);*/
GST_BOILERPLATE_FULL (GstI2C, gst_i2c, GstBaseTransform,
    GST_TYPE_BASE_TRANSFORM, _do_init);

static void gst_i2c_set_property (GObject * object, guint prop_id,
    const GValue * value, GParamSpec * pspec);
static void gst_i2c_get_property (GObject * object, guint prop_id,
    GValue * value, GParamSpec * pspec);

static void gst_i2c_finalize (GObject *object);

static gboolean gst_i2c_transform_start  (GstBaseTransform *trans);

static gboolean gst_i2c_set_caps (GstPad * pad, GstCaps * caps);
static GstFlowReturn gst_i2c_chain (GstPad * pad, GstBuffer * buf);

/* GObject vmethod implementations */

static void
gst_i2c_base_init (gpointer gclass)
{
  GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);

  gst_element_class_set_details_simple(element_class,
    "I2C",
    "FIXME:Generic",
    "FIXME:Generic Template Element",
    "Christian Hemp <<user@hostname.org>>");

  gst_element_class_add_pad_template (element_class,
      gst_static_pad_template_get (&src_factory));
  gst_element_class_add_pad_template (element_class,
      gst_static_pad_template_get (&sink_factory));
}

/* initialize the i2c's class */
static void
gst_i2c_class_init (GstI2CClass * klass)
{
  GObjectClass *gobject_class;
//  GstElementClass *gstelement_class;
  GstBaseTransformClass *trans_class;

  gobject_class = (GObjectClass *) klass;
  //gstelement_class = (GstElementClass *) klass;
//  trans_class = (GstBaseTransformClass *) klass;

  gobject_class->set_property = gst_i2c_set_property;
  gobject_class->get_property = gst_i2c_get_property;
  gobject_class->finalize = gst_i2c_finalize;

  g_object_class_install_property (gobject_class, ARG_DEV,
    g_param_spec_string ("dev", "Device",
      "Device Name (Default: /dev/video0)", "/dev/video0",
      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));

  g_object_class_install_property (gobject_class, ARG_ADDR,
    g_param_spec_string ("addr", "Chip Address",
      "Chip Address as Hex-Value (Default: 5D)", "5D",
      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));

  g_object_class_install_property (gobject_class, ARG_REG,
    g_param_spec_string ("reg", "Register",
      "Register number as Hex-Value (Default: 20)", "20",
      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));

  g_object_class_install_property (gobject_class, ARG_MODE,
    g_param_spec_enum ("mode", "Access Mode",
    "set, and, or, xor (Default: set)", GST_TYPE_I2C_ACCMODE,
    GST_I2C_ACCESSMODE_SET,
    G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));

  g_object_class_install_property (gobject_class, ARG_VALUE,
    g_param_spec_string ("value", "Value",
      "Hex-Value (Default: 0000)", "0",
      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));

  g_object_class_install_property (gobject_class, ARG_FILE,
    g_param_spec_string ("file", "File",
      "Filename (Default: none)", NULL,
      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));

  g_object_class_install_property (gobject_class, ARG_SHOW,
    g_param_spec_boolean ("show", "Show Accesses",
      "Show accesses of i2c (Default: true)", TRUE,
      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));

  g_object_class_install_property (gobject_class, PROP_SILENT,
      g_param_spec_boolean ("silent", "Silent", "Produce verbose output ?",
          FALSE, G_PARAM_READWRITE));

  trans_class = GST_BASE_TRANSFORM_CLASS (klass);
  trans_class->start = gst_i2c_transform_start;
  trans_class->passthrough_on_same_caps = TRUE;
}

/* initialize the new element
 * instantiate pads and add them to element
 * set pad calback functions
 * initialize instance structure
 */
static void
gst_i2c_init (GstI2C * filter,
    GstI2CClass * gclass)
{
  filter->sinkpad = gst_pad_new_from_static_template (&sink_factory, "sink");
  gst_pad_set_setcaps_function (filter->sinkpad,
                                GST_DEBUG_FUNCPTR(gst_i2c_set_caps));
  gst_pad_set_getcaps_function (filter->sinkpad,
                                GST_DEBUG_FUNCPTR(gst_pad_proxy_getcaps));
  gst_pad_set_chain_function (filter->sinkpad,
                              GST_DEBUG_FUNCPTR(gst_i2c_chain));

  filter->srcpad = gst_pad_new_from_static_template (&src_factory, "src");
  gst_pad_set_getcaps_function (filter->srcpad,
                                GST_DEBUG_FUNCPTR(gst_pad_proxy_getcaps));

  gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
  gst_element_add_pad (GST_ELEMENT (filter), filter->srcpad);
  filter->silent = FALSE;
  
  strcpy (filter->dev, "/dev/video0");
  filter->addr = 0x5d;
  filter->reg = 0x20;
  filter->mode = GST_I2C_ACCESSMODE_SET;
  filter->value = 0L;
  filter->file [0] = 0;
  filter->show = TRUE;
}

static unsigned long int 
parse (gchar *s) {
  unsigned long int i;
  char *p;

  p = s;
  while (*p != 0) {
    if ((*p == 'x') || (*p == 'X')) {
      sscanf (s, "%lx", &i);
      return i;
    }

    p++;
  }

  sscanf (s, "%lu", &i);
  return i;
}


static void
gst_i2c_set_property (GObject * object, guint prop_id,
    const GValue * value, GParamSpec * pspec)
{
  GstI2C *filter = GST_I2C (object);

  switch (prop_id) {
    case ARG_DEV:
      strcpy (filter->dev, g_value_get_string (value));
      break;
    case ARG_ADDR:
      filter->addr = parse (g_value_get_string (value));
      break;
    case ARG_REG:
      filter->reg = parse (g_value_get_string (value));
      break;
    case ARG_MODE:
      filter->mode = g_value_get_enum (value);
      break;
    case ARG_VALUE:
      filter->value = parse (g_value_get_string (value));
      break;
    case ARG_FILE:
      strcpy (filter->file, g_value_get_string (value));
      break;
    case ARG_SHOW:
      filter->show = g_value_get_boolean (value);
      break;
    case PROP_SILENT:
      filter->silent = g_value_get_boolean (value);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
  }
}

gchar *unparse (unsigned long int i, int len) {
  static char s [32];

  sprintf (s, "0x%0*lx", len, i);
 
  return s;
}


static void
gst_i2c_get_property (GObject * object, guint prop_id,
    GValue * value, GParamSpec * pspec)
{
  GstI2C *filter = GST_I2C (object);

  switch (prop_id) {
    case ARG_DEV:
      g_value_set_string (value, filter->dev);
      break;
    case ARG_ADDR:
      g_value_set_string (value, unparse (filter->addr, 2));
      break;
    case ARG_REG:
      g_value_set_string (value, unparse (filter->reg, 4));
      break;
    case ARG_MODE:
      g_value_set_enum (value, filter->mode);
      break;
    case ARG_VALUE:
      g_value_set_string (value, unparse (filter->value, 4));
      break;
    case ARG_FILE:
      g_value_set_string (value, filter->file);
      break;
    case ARG_SHOW:
      g_value_set_boolean (value, filter->show);
      break;

    case PROP_SILENT:
      g_value_set_boolean (value, filter->silent);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
  }
}

/*static char*
read_config_file(GstI2C *i2c)
{
  char buffer[256];
  char *p, *pp;

  while (fgets (buffer, sizeof (buffer), i2c->f) != NULL){
    p = strchr (buffer, '/');
    pp = strchr (buffer, '#');
    if ((p == NULL) || (p > pp))
      p = pp;

    if (p != NULL){
      do {
        *p = 0;
        if ( p > buffer)
          p--;
      } while (*p == ' ');
    }

    p = strchr(buffer, ',');
    if (p == NULL)  //Leerzeilen überlesen
      continue;

    *p++ = 0;
    i2c->addr = parse (buffer);
    
    pp = strchr (p, ',');
    if (pp == NULL)
      break;

    *pp++ = 0;
    i2c->reg = parse (p);

    i2c->mode = GST_I2C_ACCESSMODE_SET;
    p = strchr (pp, ',');
    if (p != NULL) {
      *p++ = 0;

      switch (*p & 0x5F) {
        case 'A': i2c->mode = GST_I2C_ACCESSMODE_AND; break;
        case 'O': i2c->mode = GST_I2C_ACCESSMODE_OR;  break;
        case 'X': i2c->mode = GST_I2C_ACCESSMODE_XOR; break;
        default:  i2c->mode = GST_I2C_ACCESSMODE_SET;
      }
    }

    i2c->value = parse (pp);
  }
}*/

static gboolean gst_i2c_transform_start  (GstBaseTransform *trans) {
  GstI2C *i2c;
  char buffer [256];
  char zeile [128];
  char zeile2 [16];
  struct v4l2_dbg_register reg;
  char *p;
  char *pp;
  int ret;

  i2c = GST_I2C (trans);
  
  // open i2c
  g_print ("i2c open %s\n", i2c -> dev);
  fflush (stdout);
  usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub

  i2c -> fd = open (i2c -> dev, O_RDWR);    // Meldungen des Kernels werden sofort rausgehauen!
  if (i2c -> fd < 0) {
    g_print ("ERROR i2c open %s\n", i2c -> dev);
    fflush (stdout);
    usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub
    return FALSE;
  }

  // Eingabefile öffnen, falls vorhanden
  if (i2c -> file [0] == 0)
    i2c -> f = NULL;
  else {
    i2c -> f = fopen (i2c -> file, "r");
    if (i2c -> f == NULL) {
      g_print ("ERROR i2c file open %s\n", i2c -> file);
      fflush (stdout);
      usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub

      i2c -> file [0] = 0;   // Damit der Finalize nicht wieder draufläuft
      return FALSE;
    }
  }

  // Eingabefile lesen, falls vorhanden
  do {
    if (i2c -> f != NULL) {
      if (fgets (buffer, sizeof (buffer), i2c -> f) == NULL)
        break;

      // Kommentare entfernen
      p = strchr (buffer, '/');
      pp = strchr (buffer, '#');
      if ((p == NULL) || (p > pp))
        p = pp;

      if (p != NULL)
        do {
          *p = 0;
          if (p > buffer)
            p--;
        } while (*p == ' ');

      // csv-Werte lesen
      p = strchr (buffer, ',');
      if (p == NULL)
        continue;     // Leerzeilen überlesen

      *p++ = 0;
      i2c -> addr = parse (buffer);

      pp = strchr (p, ',');
      if (pp == NULL){
        g_print("pp = NULL");
        break;
      }
      *pp++ = 0;
      i2c -> reg = parse (p);

      i2c -> mode = GST_I2C_ACCESSMODE_SET;
      p = strchr (pp, ',');
      if (p != NULL) {
        *p++ = 0;

        switch (*p & 0x5F) {
          case 'A': i2c -> mode = GST_I2C_ACCESSMODE_AND; break;
          case 'O': i2c -> mode = GST_I2C_ACCESSMODE_OR;  break;
          case 'X': i2c -> mode = GST_I2C_ACCESSMODE_XOR; break;
          default:  i2c -> mode = GST_I2C_ACCESSMODE_SET;
        }
      }

      i2c -> value = parse (pp);
    }

    // Ausgabezeile zusammenstellen
    strcpy (zeile, "addr ");
    strcat (zeile, unparse (i2c -> addr, 2));
    strcat (zeile, " reg ");
    strcat (zeile, unparse (i2c -> reg, 4));

    // get
    if (i2c -> show) {
      g_print ("i2c get %s\n", zeile);
      fflush (stdout);
      usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub
    }

    reg.match.type = V4L2_CHIP_MATCH_I2C_ADDR;
    reg.match.addr = i2c -> addr;
    reg.reg = i2c -> reg;
    reg.val = 0x00;
    reg.size = 0;

    ret = ioctl (i2c -> fd, VIDIOC_DBG_G_REGISTER, &reg);   // Meldungen des Kernels werden sofort rausgehauen!a

    if (ret) {
      ret = errno;
      g_print ("ERROR i2c get (ioctl) %s: %d %s\n", zeile, ret, strerror (ret));
      fflush (stdout);
      usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub
      return FALSE;
    }

    if (i2c -> show) {
      g_print ("i2c get: 0x%04x (Size: %u Bytes)\n", (__u32) reg.val, reg.size);
      fflush (stdout);
      usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub
    }

    // set/and/or/xor
    switch (i2c -> mode) {
      case GST_I2C_ACCESSMODE_SET: strcpy (zeile2, "set");
                                   reg.val =  i2c -> value;
                                   break;
      case GST_I2C_ACCESSMODE_AND: strcpy (zeile2, "and"); reg.val &= i2c -> value; break;
      case GST_I2C_ACCESSMODE_OR:  strcpy (zeile2, "or");  reg.val |= i2c -> value; break;
      case GST_I2C_ACCESSMODE_XOR: strcpy (zeile2, "xor"); reg.val ^= i2c -> value; break;
    }

    if (i2c -> show) {
      g_print ("i2c %s %s", zeile2, unparse (i2c -> value, 4));
      if (i2c -> mode != GST_I2C_ACCESSMODE_SET)
        g_print (": %s", unparse (reg.val, 4));
      g_print ("\n");
      fflush (stdout);
      usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub
    }

    ret = ioctl (i2c -> fd, VIDIOC_DBG_S_REGISTER, &reg);   // Meldungen des Kernels werden sofort rausgehauen!
    if (ret) {
      ret = errno;
      g_print ("ERROR i2c set (ioctl) %s: %d %s\n", zeile, ret, strerror (ret));
      fflush (stdout);
      usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub
      return FALSE;
    }
// get
    if (i2c -> show) {
      g_print ("i2c get %s\n", zeile);
      fflush (stdout);
      usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub

      reg.match.type = V4L2_CHIP_MATCH_I2C_ADDR;
      reg.val = 0x00;
      reg.size = 0;
 
      ret = ioctl (i2c -> fd, VIDIOC_DBG_G_REGISTER, &reg);   // Meldungen des Kernels werden sofort rausgehauen!
 
      if (ret) {
        ret = errno;
        g_print ("ERROR i2c get (ioctl) %s: %d %s\n", zeile, ret, strerror (ret));
        fflush (stdout);
        usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub
        return FALSE;
      }

      g_print ("i2c get: 0x%04x (Size: %u Bytes)\n", (__u32) reg.val, reg.size);
      fflush (stdout);
      usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub
    }

  // Ende Eingabefile lesen, falls vorhanden
  } while (i2c -> f != NULL);

  // Eingabefile schließen, falls vorhanden
  if (i2c -> f != NULL)
    fclose (i2c -> f);

  return TRUE;
}

static void gst_i2c_finalize (GObject *object) {
  GstI2C *i2c;
  char buffer [256];
  char zeile [128];
  struct v4l2_dbg_register reg;
  char *p;
  char *pp;
  int ret;

  i2c = GST_I2C (object);

  if (i2c -> show) {

    // Eingabefile öffnen, falls vorhanden
    if (i2c -> file [0] != 0) {
      i2c -> f = fopen (i2c -> file, "r");
      if (i2c -> f == NULL) {
        g_print ("ERROR i2c file open %s\n", i2c -> file);
        fflush (stdout);
        usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub
        goto raus;
      }
    }

    // Eingabefile lesen, falls vorhanden
    do {
      if (i2c -> f != NULL) {
        if (fgets (buffer, sizeof (buffer), i2c -> f) == NULL)
          break;

        // Kommentare entfernen
        p = strchr (buffer, '/');
        pp = strchr (buffer, '#');
        if ((p == NULL) || (p > pp))
          p = pp;

        if (p != NULL)
          do {
            *p = 0;
            if (p > buffer)
              p--;
          } while (*p == ' ');

        // csv-Werte lesen
        p = strchr (buffer, ',');
        if (p == NULL){
          continue;     // Leerzeilen überlesen
        }
        *p++ = 0;
        i2c -> addr = parse (buffer);

        pp = strchr (p, ',');
        if (pp == NULL)
          break;
        *pp++ = 0;
        i2c -> reg = parse (p);

        i2c -> mode = GST_I2C_ACCESSMODE_SET;
        p = strchr (pp, ',');
        if (p != NULL) {
          *p++ = 0;

          switch (*p & 0x5F) {
            case 'A': i2c -> mode = GST_I2C_ACCESSMODE_AND; break;
            case 'O': i2c -> mode = GST_I2C_ACCESSMODE_OR;  break;
            case 'X': i2c -> mode = GST_I2C_ACCESSMODE_XOR; break;
            default:  i2c -> mode = GST_I2C_ACCESSMODE_SET;
          }
        }

        i2c -> value = parse (pp);
      }

      // Ausgabezeile zusammenstellen
      strcpy (zeile, "addr ");
      strcat (zeile, unparse (i2c -> addr, 2));
      strcat (zeile, " reg ");
      strcat (zeile, unparse (i2c -> reg, 4));

      // get
      g_print ("i2c get %s\n", zeile);
      fflush (stdout);
      usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub

      reg.match.type = V4L2_CHIP_MATCH_I2C_ADDR;
      reg.match.addr = i2c -> addr;
      reg.reg = i2c -> reg;
      reg.val = 0x00;
      reg.size = 0;

      ret = ioctl (i2c -> fd, VIDIOC_DBG_G_REGISTER, &reg);   // Meldungen des Kernels werden sofort rausgehauen!
      if (ret) {
        ret = errno;
        g_print ("ERROR i2c get (ioctl) %s: %d %s\n", zeile, ret, strerror (ret));
        fflush (stdout);
        usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub
      }

      g_print ("i2c get: 0x%04x (Size: %u Bytes)\n", (__u32) reg.val, reg.size);
      fflush (stdout);
      usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub

    // Ende Eingabefile lesen, falls vorhanden
    } while (i2c -> f != NULL);

    // Eingabefile schließen, falls vorhanden
  if (i2c -> f != NULL)
      fclose (i2c -> f);

  }

raus:

  // close i2c
  g_print ("i2c close\n");
  fflush (stdout);
  usleep (9000);   // Der Kernel wartet sonst trotz Flush nur für die Ausgabe bis vor den Zeilenvorschub

  close (i2c -> fd);   // Meldungen des Kernels werden sofort rausgehauen!

  G_OBJECT_CLASS (parent_class) -> finalize (object);
}


/* GstElement vmethod implementations */

/* this function handles the link with other elements */
static gboolean
gst_i2c_set_caps (GstPad * pad, GstCaps * caps)
{
  GstI2C *filter;
  GstPad *otherpad;

  filter = GST_I2C (gst_pad_get_parent (pad));
  otherpad = (pad == filter->srcpad) ? filter->sinkpad : filter->srcpad;
  gst_object_unref (filter);

  return gst_pad_set_caps (otherpad, caps);
}

/* chain function
 * this function does the actual processing
 */
static GstFlowReturn
gst_i2c_chain (GstPad * pad, GstBuffer * buf)
{
  GstI2C *filter;

  filter = GST_I2C (GST_OBJECT_PARENT (pad));

  if (filter->silent == FALSE)
    g_print ("I'm plugged, therefore I'm in.\n");

  /* just push out the incoming buffer without touching it */
  return gst_pad_push (filter->srcpad, buf);
}


/* entry point to initialize the plug-in
 * initialize the plug-in itself
 * register the element factories and other features
 */
static gboolean
i2c_init (GstPlugin * i2c)
{
  /* debug category for fltering log messages
   *
   * exchange the string 'Template i2c' with your description
   */
  GST_DEBUG_CATEGORY_INIT (gst_i2c_debug, "i2c",
      0, "Template i2c");
 
  return gst_element_register (i2c, "i2c", GST_RANK_NONE,
      GST_TYPE_I2C);
}

/* PACKAGE: this is usually set by autotools depending on some _INIT macro
 * in configure.ac and then written into and defined in config.h, but we can
 * just set it ourselves here in case someone doesn't use autotools to
 * compile this code. GST_PLUGIN_DEFINE needs PACKAGE to be defined.
 */
#ifndef PACKAGE
#define PACKAGE "myfirsti2c"
#endif

/* gstreamer looks for this structure to register i2cs
 *
 * exchange the string 'Template i2c' with your i2c description
 */
GST_PLUGIN_DEFINE (
    GST_VERSION_MAJOR,
    GST_VERSION_MINOR,
    "i2c",
    "Template i2c",
    i2c_init,
    VERSION,
    "LGPL",
    "GStreamer",
    "http://gstreamer.net/"
)
