/*
 * Copyright (C) 2012 Michael Olbrich <m.olbrich@pengutonix.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */

#ifndef CPU_USAGE_H
#define CPU_USAGE_H

#include <QtCore/QObject>
#include <QtCore/QVector>

class CpuUsage : public QObject
{
	Q_OBJECT
	Q_PROPERTY(int interval READ interval WRITE setInterval NOTIFY intervalChanged)
	Q_PROPERTY(int core READ core WRITE setCore NOTIFY coreChanged)
	Q_PROPERTY(int user READ user NOTIFY userChanged)
	Q_PROPERTY(int nice READ nice NOTIFY niceChanged)
	Q_PROPERTY(int system READ system NOTIFY systemChanged)
	Q_PROPERTY(int idle READ idle NOTIFY idleChanged)
	Q_PROPERTY(int all READ all NOTIFY allChanged)

public:
	CpuUsage(QObject *parent = 0);
	~CpuUsage();

	int interval() const;
	int core() const;

	int user() const;
	int nice() const;
	int system() const;
	int idle() const;
	int all() const;

public slots:
	void setInterval(int interval);
	void setCore(int core);

signals:
	void intervalChanged(int interval);
	void coreChanged(int core);
	void userChanged(int user);
	void niceChanged(int nice);
	void systemChanged(int system);
	void idleChanged(int idle);
	void allChanged(int all);

protected:
	virtual void timerEvent(QTimerEvent *);

private:
	void getValues();

private:
	int m_id;
	int m_interval;
	int m_core;
	QVector<int> m_data, m_old;
	int m_now;
};

#endif // CPU_USAGE_H
